@@ -38,7 +38,7 @@ def totalMassBalanceValue(totalMB, area, res):
 		res: pixel resolution (int or float)
     """
     # totalMB[totalMB == 0] = np.nan
-    totalMassBalanceVal = np.sum(totalMB * res * res) / area  # total mass balance in kg/m2-yr
+    totalMassBalanceVal = np.nansum(totalMB * res * res) / area  # total mass balance in kg/m2-yr
     return totalMassBalanceVal
 
 def divQ(vx, vy, h, res, vCol=0.8):
@@ -52,6 +52,21 @@ def divQ(vx, vy, h, res, vCol=0.8):
     divQarray = ((np.gradient(vx, res)[1] * h) + (np.gradient(h, res)[1] * vx) +
                   (np.gradient(vy * -1, res)[0] * h) + (np.gradient(h * -1, res)[0] * vy)) * vCol
     return divQarray
+    
+def flux_div_comps(vx, vy, h, res, vCol=0.8):
+   """
+	Flux divergence - return each component as well
+		vx, vy: x- and y-direction velocity, m/yr (array-like)
+		h: glacier thickness, m (array-like)
+		res: pixel resolution (int or float)
+		vCol: vertical-average velocity scaling factor, typically 0.8 (int or float)
+    """
+    dvx = (np.gradient(vx, res)[1] * h) * vCol
+    dvy = (np.gradient(vy * -1, res)[0] * h) * vCol 
+    dhx = (np.gradient(h, res)[1] * vx) * vCol
+    dhy = (np.gradient(h * -1, res)[0] * vy) * vCol
+    divQ = dvx + dvy + dhx + dhy
+    return divQ, dvx, dvy, dhx, dhy
 
 
 def glacierAttributes(dem_rast, attrFormat):
@@ -87,6 +102,29 @@ def demHillshade(demArray, az=135, alt=30):
     # altitude is from 0 to 90, 90 being directly overhead
     hillshade = es.hillshade(demArray, azimuth=az, altitude=alt)     # Create and plot the hillshade with earthpy
     return hillshade
+    
+def demAspect(dem_array, res):
+    """
+    Aspect of a DEM array – takes an array input instead of raster
+	dem_array: input array (array-like)
+	res: pixel resolution (int or float)
+    """
+    aspect_array = np.zeros_like(dem_array)
+    py, px = np.gradient(dem_array, res)
+    print(px)
+    print(py)
+
+    for i in range(len(aspect_array)):
+        for j in range(len(aspect_array[0])):
+            pixel_deg = math.atan2(py[i][j], px[i][j]) * 180 / math.pi
+            if pixel_deg >= 0 and pixel_deg <= 90:
+                pixel_aspect = 90 - pixel_deg
+            elif pixel_deg < 0:
+                pixel_aspect = abs(pixel_deg) + 90
+            elif pixel_deg > 90:
+                pixel_aspect = 450 - pixel_deg
+            aspect_array[i][j] = pixel_aspect
+    return aspect_array
 
 
 def velocityAspect(vel_x, vel_y):
